@@ -1,5 +1,5 @@
 package Sim::OPT::Descend;
-# Copyright (C) 2008-2017 by Gian Luca Brunetti and Politecnico di Milano.
+# Copyright (C) 2008-2018 by Gian Luca Brunetti and Politecnico di Milano.
 # This is the module Sim::OPT::Descend of Sim::OPT, a program for detailed metadesign managing parametric explorations through the ESP-r building performance simulation platform and performing optimization by block coordinate descent.
 # This is free software.  You can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 3.
 
@@ -36,7 +36,7 @@ no warnings;
 @ISA = qw(Exporter); # our @adamkISA = qw(Exporter);
 #%EXPORT_TAGS = ( DEFAULT => [qw( &opt &prepare )]); # our %EXPORT_TAGS = ( 'all' => [ qw( ) ] );
 #@EXPORT   = qw(); # our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
-our @EXPORT = qw( descend sense ); # our @EXPORT = qw( );
+our @EXPORT = qw( descend prepareblank ); # our @EXPORT = qw( );
 
 $VERSION = '0.121'; # our $VERSION = '';
 $ABSTRACT = 'Sim::OPT::Descent is an module collaborating with the Sim::OPT module for performing block coordinate descent.';
@@ -117,7 +117,7 @@ sub descend
 
   my $exitname = $dirfiles{exitname};
 
-  say $tee "IN ENTRY DESCEND3  \$dirfiles{starsign} " . dump( $dirfiles{starsign} );
+  #say $tee "IN ENTRY DESCEND3  \$dirfiles{starsign} " . dump( $dirfiles{starsign} );
 
   my %d = %{ $instances[0] }; #say $tee "IN ENTRY DESCEND3 \%d : " . dump( \%d );
   my $countcase = $d{countcase}; #say $tee "IN ENTRY DESCEND \$countcase : " . dump( $countcase );
@@ -145,7 +145,7 @@ sub descend
     {
       Sim::OPT::sense( $dirfiles{ordtot}, $mypath, $dowhat{objectivecolumn} );
     }
-    exit(say $tee "#END RUN.");
+    exit(say $tee "3 #END RUN.");
   }
 
 	my @blockelts = @{ $d{blockelts} }; #say $tee "IN ENTRY DESCEND \@blockelts : " . dump( @blockelts );
@@ -223,7 +223,7 @@ sub descend
   my $tottot = $dirfiles{tottot}; #say $tee "IN ENTRY DESCEND \$tottot: " . dump( $tottot );
   my $ordtot = $dirfiles{ordtot}; #say $tee "IN ENTRY DESCEND \$ordtot: " . dump( $ordtot );
 
-  my $confinterlinear = "$mypath/" . $dowhat{confinterlinear} ; say $tee "IN DESCEND5 \$confinterlinear: " . dump( $confinterlinear );
+  my $confinterlinear = "$mypath/" . $dowhat{confinterlinear} ; #say $tee "IN DESCEND5 \$confinterlinear: " . dump( $confinterlinear );
 
   my $repfile = $dirfiles{repfile}; #say $tee "IN DESCEND \$repfile: " . dump( $repfile );
   if ( not( $repfile ) ){ die; }
@@ -284,10 +284,10 @@ sub descend
     {
       Sim::OPT::sense( $dirfiles{ordtot}, $mypath, $dowhat{objectivecolumn} );
     }
-    exit(say $tee "#END RUN.");
+    exit(say $tee "4 #END RUN.");
   }
 
-  say $tee " \$repfile " . dump($repfile);
+  #say $tee " \$repfile " . dump($repfile);
   if ( not( -e $repfile ) ){ die "There isn't \$repfile: $repfile"; };
 
 
@@ -749,7 +749,7 @@ sub descend
       my $thoselines;
       if ( ( $entryfile ne "" ) and ( -e $entryfile ) )
       {
-        open( ENTRYFILE, "$entryfile" ) or die; say $tee "IN DESCENT OPENING \$entryfile" . dump( $entryfile );
+        open( ENTRYFILE, "$entryfile" ) or die; #say $tee "IN DESCENT OPENING \$entryfile" . dump( $entryfile );
         $thoselines = <ENTRYFILE>;
         close ENTRYFILE;
       }
@@ -818,7 +818,8 @@ sub descend
 
   sub metamodel
 	{
-    my ( $dowhat_r, $sortmixed, $file, $dirfiles_r, $blockelts_r, $carrier_r, $metafile, $direction, $starorder, $ordmeta, $varnums_r, $countblock ) = @_;
+    my ( $dowhat_r, $sortmixed, $file, $dirfiles_r, $blockelts_r, $carrier_r, $metafile,
+      $direction, $starorder, $ordmeta, $varnums_r, $countblock ) = @_;
     my %dowhat = %{ $dowhat_r };
     my %dirfiles = %{ $dirfiles_r };
     my @blockelts = @{ $blockelts_r }; #say $tee " IN metamodel \@blockelts : " . dump( @blockelts );
@@ -828,7 +829,7 @@ sub descend
     close SORTMIXED;
 
 	  #my ( $dowhat_r, $sortmixed, $file, $dirfiles_r, $blockelts_r, $mids_r, $metafile, $direction, $starorder, $ordmeta ) = @_;
-		#say $tee " IN TAKEOPTIMA \$dowhat{metamodel} : " . dump( $dowhat{metamodel} );
+		#say $tee " IN metamodel \$dowhat{metamodel} : " . dump( $dowhat{metamodel} );
 		if ( $dowhat{metamodel} eq "yes" ) # PART ON METAMODELS
 		{
 		  sub cleanres
@@ -854,107 +855,27 @@ sub descend
       sub prepareblank
 	    {
 	      my ( $varnums_r, $blankfile, $blockelts_r, $bag_r, $file, $carrier_r ) = @_;
-	      my %varnums = %{ $varnums_r }; #say $tee "IN prepareblank \%varnums : " . dump( \%varnums );
-	      my @blockelts = ( sort { $a <=> $b } @{ $blockelts_r } ); #say $tee "IN prepareblank1 \@blockelts : " . dump( @blockelts );
-        my @bag = @{ $bag_r };
-        my %carrier = %{ $carrier_r }; #say $tee "IN prepareblank1 \%carrier : " . dump( \%carrier ); say $tee "IN prepareblank1 \%mids : " . dump( \%mids );
+	      my %varnums = %{ $varnums_r }; #say "IN prepareblank \%varnums : " . dump( \%varnums );
+	      my @blockelts = ( sort { $a <=> $b } @{ $blockelts_r } ); #say "IN prepareblank1 \@blockelts : " . dump( @blockelts );
+        my @bag = @{ $bag_r }; #say "IN prepareblank1 \@bag : " . dump( \@bag );
+        my %carrier = %{ $carrier_r }; #say "IN prepareblank1 \%carrier : " . dump( \%carrier );
         open( BLANKFILE, ">$blankfile" ) or die;
 
-        #my %checkvarnums;
-        #foreach my $key ( keys %varnums )
-        #{
-        #  if ( $key ~~ @blockelts )
-        #  {
-        #    $checkvarnums{$key} = $varnums{$key};
-        #  }
-        #  else
-        #  {
-        #    $checkvarnums{$key} = 1;
-        #  }
-        #} say $tee "IN prepareblank1 \%checkvarnums : " . dump( \%checkvarnums );
+        my @box = @{ Sim::OPT::toil( \@blockelts, \%varnums, $bag_r ) };
+	      @box = @{ $box[-1] }; #say "IN prepareblank \@box : " . dump( @box );
+	      my $integrated_r = Sim::OPT::integratebox( \@box, \%carrier, $file, \@blockelts ); #say "IN prepareblank \$integrated_r : " . dump( $integrated_r );
 
-	      #my @box = @{ Sim::OPT::toil( \@blockelts, \%checkvarnums, $bag_r ) }; say $tee "IN prepareblank \@box : " . dump( @box );
-        my @box = @{ Sim::OPT::toil( \@blockelts, \%varnums, $bag_r ) }; #say $tee "IN prepareblank \@box : " . dump( @box );
-	      @box = @{ $box[-1] };
-	      my $integrated_r = Sim::OPT::integratebox( \@box, \%carrier, $file, \@blockelts ); #say $tee "IN prepareblank \$integrated_r : " . dump( $integrated_r );
+	      my @finalbox = @{ $integrated_r }; #say "IN PREPAREBLANK \@finalbox : " . dump( @finalbox );
 
-	      #my @finalbox;
-        #
-	      #sub cleanthis
-	      #{
-	      #  my ( $integrated_r, $mypath, $file, $blockelts_r, $carrier_r ) = @_;
-        #  my @blockelts = @{ $blockelts_r };
-        #  my %carrier = %{ $carrier_r };
-	      #  foreach my $el ( @{ $integrated_r } )
-	      #  { #say $tee "IN LOOP \$el : " . dump( $el ); say $tee "IN LOOP \$el->[0] : " . dump( $el->[0] );
-	      #    $el->[0] =~ s/$mypath// ; #say $tee "IN LOOP \$el->[0] : " . dump( $el->[0] );
-	      #    $el->[0] =~ s/^\/// ; #say $tee "IN LOOP \$el->[0] : " . dump( $el->[0] );
-	      #    $el->[0] =~ s/^$file// ; #say $tee "IN LOOP \$el->[0] : " . dump( $el->[0] );
-	      #    $el->[0] =~ s/^_// ; #say $tee "IN LOOP \$e say $tee "IN LOOP \$el->[0] : " . dump( $el->[0] );l->[0] : " . dump( $el->[0] );
-	      #    $el->[0] =~ s/_$// ;
-	      #    push ( @finalbox, $el->[0] );
-	      #  }
-	      #  @finalbox = uniq( @finalbox );
-	      #  @finalbox = sort { $a <=> $b } @finalbox;
-	      #  #@finalbox = sort @finalbox; #say $tee "FINALBOX!!! : " . dump( @finalbox );
-	      #  return( @finalbox );
-	      #}
-
-	      #my @finalbox = cleanthis( $integrated_r, $mypath, $file, \@blockelts, \%carrier ); say $tee "IN PREPAREBLANK \@finalbox : " . dump( @finalbox );
-        my @finalbox = @{ $integrated_r }; #say $tee "IN PREPAREBLANK \@finalbox : " . dump( @finalbox );
-
-        #sub reclean
-        #{
-        #  my ( $finalbox_r, $blockelts_r, $carrier_r ) = @_;
-        #  my @finalbox = @{ $finalbox_r }; say $tee "RECLEAN FINALBOX!!! : " . dump( @finalbox );
-        #  my @blockelts = @{ $blockelts_r };
-        #  my %carrier = %{ $carrier_r };
-
-        #  my @bag;
-        #  my $score;
-        #  foreach my $key ( keys %carrier )
-        #  {
-        #    if ( not ( $key ~~ @blockelts ) )
-        #    {
-        #      my $fragment = "$key" . "-" . "$carrier{$key}";
-        #      push( @bag, $fragment );
-        #    }
-        #    $score = scalar( @bag );
-        #  }
-        #
-        #  my @box;
-        #  foreach my $line ( @finalbox )
-        #  {
-        #    my $hit = 1;
-        #    foreach my $fragment ( @bag )
-        #    {
-        #      if ( $line =~ /$fragment/ )
-        #      {
-        #        $hit++;
-        #      }
-        #    }
-        #    if ( $hit == $score )
-        #    {
-        #      push( @box, $line );
-        #    }
-        #  }
-        #  return( @box );
-        #}
-
-        #my @finalnewbox = reclean( \@finalbox, \@blockelts, \%carrier ); say $tee "RECLEAN FINALNEWBOX!!! : " . dump( @finalnewbox );
-
-
-	      #foreach my $el ( @finalnewbox )
         foreach my $el ( @finalbox )
 	      {
 	        say BLANKFILE $el->[0];
 	      }
 	      close BLANKFILE;
+        return( \@finalbox )
 	    }
 
 
-      #say $tee "BEFORE prepvarnums \%varnums " . dump( \%varnums );
-
       sub prepvarnums
       {
         my ( $varns_ref, $blockelts_r ) = @_;
@@ -971,8 +892,16 @@ sub descend
       }
 
       #say $tee " IN PREPAREBLANK FROM PREPVARNUMS \$dirfiles{varnumbershold}->[0] : " . dump( $dirfiles{varnumbershold}->[0] );
-      my %varns = prepvarnums( $dirfiles{varnumbershold}->[0], \@blockelts ); #say $tee " IN PREPAREBLANK FROM PREPVARNUMS \%varns : " . dump( \%varns );
+      my %varns;
+      if ( $dirfiles{starsign} eq "yes ")
+      {
+        %varns = prepvarnums( $dirfiles{varnumbershold}->[0], \@blockelts ); #say $tee " IN PREPAREBLANK FROM PREPVARNUMS \%varns : " . dump( \%varns );
       #say $tee "TO PREPAREBLANK \%carrier : " . dump( \%carrier); say $tee "\@blockelts : " . dump( @blockelts );
+      }
+      else
+      {
+        %varns = prepvarnums( \%varnums, \@blockelts ); #say $tee " IN PREPAREBLANK FROM PREPVARNUMS \%varns : " . dump( \%varns );
+      }
 
       my $cleanordres = cleanres( $sortmixed ); #say $tee "IN TAKEOPTIMA1 \$cleanordres : " . dump( $cleanordres );
 	    my $blankfile = "$sortmixed" . "_tmp_blank.csv";
@@ -1044,7 +973,8 @@ sub descend
       my $prepsort = $sortmixed . "_tmp_prepsort.csv";
       #say $tee "BEFORE PREPFILE \%carrier " . dump( \%carrier );
       #say $tee "BEFORE PREPFILE \%varnums " . dump( \%varnums );
-      my %torecovers = prepfile( $blankfile, $prepblank, $cleanordres, $prepsort, \@blockelts, \%varnums, \%carrier, $metafile ); say $tee "FROM PREPFILE \%torecovers " . dump( \%torecovers );
+      my %torecovers = prepfile( $blankfile, $prepblank, $cleanordres, $prepsort, \@blockelts, \%varnums, \%carrier, $metafile );
+      #say $tee "FROM PREPFILE \%torecovers " . dump( \%torecovers );
 
 
       #say $tee "BEFORE jointwo \%varnums " . dump( \%varnums );
@@ -1090,7 +1020,7 @@ sub descend
 	    jointwo( $prepsort, $prepblank, $prepfile ) ;
 
       my $rawmetafile = $metafile . "_tmp_raw.csv";
-	    #say $tee "ABOUT TO LAUNCH INTERLINEAR";
+	    say $tee "ABOUT TO LAUNCH INTERLINEAR";
 	    #say $tee "WITH, IN TAKEOPTIMA \$confinterlinear: " . dump($confinterlinear);
       #say $tee "WITH, IN TAKEOPTIMA \$prepfile: " . dump($prepfile);
 	    #say $tee "WITH, IN TAKEOPTIMA \$metafile: " . dump($metafile);
@@ -1242,7 +1172,11 @@ sub descend
       #}
 
       #say $tee "IN CASE END."; say $tee "\$dirfiles{starsign} " . dump( $dirfiles{starsign} );
-      if ( $dirfiles{starsign} eq "yes" ) ### BEGINNING OF THE PART ON STAR CONFIGURATIONS
+      if ( ( $dirfiles{starsign} eq "yes" ) or
+    			( ( $dirfiles{random} eq "yes" ) and ( $dowhat{metamodel} eq "yes" ) ) or
+    			( ( $dirfiles{latinhypercube} eq "yes" ) and ( $dowhat{metamodel} eq "yes" ) ) or
+    			( ( $dirfiles{factorial} eq "yes" ) and ( $dowhat{metamodel} eq "yes" ) ) or
+    			( ( $dirfiles{facecentered} eq "yes" ) and ( $dowhat{metamodel} eq "yes" ) ) ) ### BEGINNING OF THE PART ON STAR CONFIGURATIONS
       {
         #say $tee "IN CASE END AND STARSIGN.";
         #say $tee "NOT YET.";
@@ -1412,10 +1346,11 @@ sub descend
 
         #say $tee "CARRIER " . dump( %carrier );
 
-        my @blockelts = @{ Sim::OPT::getblockelts( \@sweeps, $countcase, $countblock ) }; say $tee "IN callblock \@blockelts " . dump( @blockelts );
+        my @blockelts = @{ Sim::OPT::getblockelts( \@sweeps, $countcase, $countblock ) }; #say $tee "IN callblock \@blockelts " . dump( @blockelts );
 
         #say $tee "ABOUT TO CALL METAMODEL WITH COUNT EQUAL TO MAX; \$countblock: $countblock";
-        metamodel( \%dowhat, $sortmixed, $file, \%dirfiles, \@blockelts, \%carrier, $metafile, $direction, $starorder, $ordmeta, \%varnums, $countblock );
+        metamodel( \%dowhat, $sortmixed, $file, \%dirfiles, \@blockelts, \%carrier, $metafile,
+          $direction, $starorder, $ordmeta, \%varnums, $countblock );
 
         my @lines;
         if ( $dirfiles{metamodel} eq "yes" )
@@ -1469,7 +1404,7 @@ sub descend
           {
             Sim::OPT::sense( $dirfiles{ordtot}, $mypath, $dowhat{objectivecolumn} );
           }
-          exit(say $tee "#END RUN.");
+          exit(say $tee "1 #END RUN.");
         }
 
         $countstring++;
@@ -1503,7 +1438,7 @@ sub descend
           {
             Sim::OPT::sense( $dirfiles{ordtot}, $mypath, $dowhat{objectivecolumn} );
           }
-          exit(say $tee "#END RUN.");
+          exit(say $tee "2 #END RUN.");
         }
 
         $countstring++;
@@ -1543,8 +1478,12 @@ sub descend
       #  $dirfiles{starsign} = "yes"; #say $tee "SETTING IN callblock: \$dirfiles{starsign} " . dump( $dirfiles{starsign} );
       #}
 
-      if ( $dirfiles{starsign} eq "yes" ) ### BEGINNING OF THE PART ABOUT STAR CONFIGURATIONS
-      {
+      if ( ( $dirfiles{starsign} eq "yes" ) or
+    			( ( $dirfiles{random} eq "yes" ) and ( $dowhat{metamodel} eq "yes" ) ) or
+    			( ( $dirfiles{latinhypercube} eq "yes" ) and ( $dowhat{metamodel} eq "yes" ) ) or
+    			( ( $dirfiles{factorial} eq "yes" ) and ( $dowhat{metamodel} eq "yes" ) ) or
+    			( ( $dirfiles{facecentered} eq "yes" ) and ( $dowhat{metamodel} eq "yes" ) ) )
+      { ### BEGINNING OF THE PART ABOUT STAR CONFIGURATIONS
         #say $tee "IN BLOCK END STARSIGN.";
         #say $tee "HERE AGAIN?!";
         #say $tee "HERE \$#blockelts : " . dump( $#blockelts );
@@ -1557,7 +1496,8 @@ sub descend
         my @blockelts = @{ Sim::OPT::getblockelts( \@sweeps, $countcase, $countblock ) }; #say $tee "IN callblock \@blockelts " . dump( @blockelts );
 
         #say $tee "ABOUT TO CALL METAMODEL WITH COUNT LESSER THAN MAX; \$countblock: $countblock";
-        metamodel( \%dowhat, $sortmixed, $file, \%dirfiles, \@blockelts, \%carrier, $metafile, $direction, $starorder, $ordmeta, \%varnums, $countblock );
+        metamodel( \%dowhat, $sortmixed, $file, \%dirfiles, \@blockelts, \%carrier, $metafile,
+          $direction, $starorder, $ordmeta, \%varnums, $countblock );
 
         my @lines;
         if ( $dirfiles{metamodel} eq "yes" )
